// saleem, Oct 2002
// saleem, Dec 2001
// example usage of netbyte package

import netbyte.*;

public class NetByteDemo
{
    // this is my packet header
    //         8       16      24      32  bits
    //  +------+-------+-------+-------+
    //  | ver  | type  |       s       |
    //  +------+-------+-------+-------+
    //  |             i1               |
    //  +------+-------+-------+-------+
    //  |             i2               |
    //  +------+-------+-------+-------+
    //  |              d               |
    //  |                              |
    //  +------+-------+-------+-------+

    // these are the header fields
    byte ver_;
    byte type_;
    short s_;
    int i1_;
    int i2_;
    double d_;

    // the size of the primitive types are:
    //
    //   byte   1  (of course!)
    //   char   2
    //   short  2
    //   int    4
    //   long   8
    //   float  4
    //   double 8
    //
    // if you are using float and double in a packet header then you
    // you are probably doing something strange! I have put in
    // float and double for the sake of completeness ... and of course
    // you may (for whatever reason) want to use these in your data


    // offsets in the pkt header, the byte
    // index where the header field starts
    final static int ver_Offset = 0;
    final static int type_Offset = 1;
    final static int s_Offset = 2;
    final static int i1_Offset = 4;
    final static int i2_Offset = 8;
    final static int d_Offset = 12;

    // this is the number of bytes I need for my header
    int _size = // should be 20
        NetByte.byteSize_ + // ver_
        NetByte.byteSize_ + // type_
        NetByte.shortSize_ + // _s
        NetByte.intSize_ + // i1_
        NetByte.intSize_ + // i2_
        NetByte.doubleSize_; // d_

    static public void main(String[] argv)
    {
        String hex = null;
        NetByteDemo m1, m2;

        System.out.println("Original packet ...");
        m1 = new NetByteDemo((byte) 0x01, (byte) 0x06, (short) 15, 42, 1, 
                              6.9);
        m1.print();
        System.out.println("");

        System.out.println("Encoded byte stream ...");
        byte[] pkt = m1.encode();
        hex = NetByte.bytesToHexString(pkt, pkt.length, 0);
        System.err.println("pkt length: " + pkt.length + " bytes");
        System.out.println("pkt encoding: " + hex);
        System.out.println("");

        System.out.println("New packet made from encoded byte stream ...");
        m2 = new NetByteDemo(pkt);
        m2.print();
    }


    NetByteDemo(byte ver, byte type, short s, int i1, int i2, double d)
    {
        ver_ = ver;
        type_ = type;
        s_ = s;
        i1_ = i1;
        i2_ = i2;
        d_ = d;
    }


    NetByteDemo(byte[] buffer)
    {
        if (buffer.length < _size) {
            System.err.println("aaargh!");
            System.exit(-1);
        }

        ver_ = buffer[ver_Offset];
        type_ = buffer[type_Offset];
        try {
            s_ = NetByte.decodeShort(buffer, s_Offset);
            i1_ = NetByte.decodeInt(buffer, i1_Offset);
            i2_ = NetByte.decodeInt(buffer, i2_Offset);
            d_ = NetByte.decodeDouble(buffer, d_Offset);
        }
        catch (NetByteException e) {
            String eName = e.getClass().getName();
            String eMsg = e.getMessage();
            System.err.println("oh dear ... " + eName + " " + eMsg);
        }
    }


    byte[] encode()
    {
        byte[] buffer = new byte[_size];

        buffer[ver_Offset] = ver_;
        buffer[type_Offset] = type_;
        try {
            NetByte.encodeShort(s_, buffer, s_Offset);
            NetByte.encodeInt(i1_, buffer, i1_Offset);
            NetByte.encodeInt(i2_, buffer, i2_Offset);
            NetByte.encodeDouble(d_, buffer, d_Offset);
        }
        catch (NetByteException e) {
            String eName = e.getClass().getName();
            String eMsg = e.getMessage();
            System.err.println("oh dear ... " + eName + " " + eMsg);
        }

        return buffer;
    }


    void print()
    {
        System.out.println("ver_: " + ver_);
        System.out.println("type_: " + type_);
        System.out.println("s_: " + s_);
        System.out.println("i1_: " + i1_);
        System.out.println("i2_: " + i2_);
        System.out.println("d_: " + d_);        
    }
}
